@@ -10,6 +10,7 @@ import cairo
 import time
 import hildon
 import gobject
+import re
 
 from threading import Thread
 
@@ -131,23 +132,31 @@ class googleRoutesearchDownloader(Thread):
             sll = [ sl.getElementsByTagName("lat")[0].childNodes[0].data, sl.getElementsByTagName("lng")[0].childNodes[0].data]
             ell = [ el.getElementsByTagName("lat")[0].childNodes[0].data, el.getElementsByTagName("lng")[0].childNodes[0].data]
 
-            str = step.getElementsByTagName("html_instructions")[0].childNodes[0].data.replace("<b>","").replace("</b>","")
+            str = re.sub(r'<[^>]*?>', ' ', step.getElementsByTagName("html_instructions")[0].childNodes[0].data )
+            str = str.replace("  ", " ")
             ico = None
+            ico_name = None
 
             if str.find("Continue straight") == 0:
-                ico = gdk.pixbuf_new_from_file("./plugins/ico_turn_32_32.png")
+                ico_name = "./plugins/ico_turn_32_32.png"
+                ico = gdk.pixbuf_new_from_file(ico_name)
             elif str.find("Turn left") == 0:
-                ico = gdk.pixbuf_new_from_file("./plugins/ico_turn_left_32_32.png")
+                ico_name = "./plugins/ico_turn_left_32_32.png"
+                ico = gdk.pixbuf_new_from_file(ico_name)
             elif str.find("Turn right") == 0:
-                ico = gdk.pixbuf_new_from_file("./plugins/ico_turn_right_32_32.png")
+                ico_name = "./plugins/ico_turn_right_32_32.png"
+                ico = gdk.pixbuf_new_from_file(ico_name)
             elif str.find("Slight right") == 0:
-                ico = gdk.pixbuf_new_from_file("./plugins/ico_turn_sright_32_32.png")
+                ico_name = "./plugins/ico_turn_sright_32_32.png"
+                ico = gdk.pixbuf_new_from_file(ico_name)
             elif str.find("Slight left") == 0:
-                ico = gdk.pixbuf_new_from_file("./plugins/ico_turn_sleft_32_32.png")
+                ico_name = "./plugins/ico_turn_sleft_32_32.png"
+                ico = gdk.pixbuf_new_from_file( ico_name)
             self.geonames.markers.append([
                                  float(sll[0]),float(sll[1]),
                                  str,
-                                 ico
+                                 ico,
+                                 ico_name
                                  ])
 
             t = osmgpsmap.GpsMapTrack()
@@ -272,10 +281,29 @@ class google_routesearch:
 
         self.icons = []
 
+        self.my_point = -1
+
         self.zoom_minimum = 15
         self.zoom_min = 10
         self.zoom_max = 16
 
+        self.espeak_say = ""
+        self.espeak_dis = 0
+        self.espeak_time = 0
+
+        if os.path.isfile("/usr/bin/espeak"):
+            self.espeak = 1
+            print "google_routesearch - found espeak yyyy cool"
+        else:
+            self.espeak = 0
+        try:
+            print "google_routesearch speak:", self.CONFIG['google_routesearch_speak']
+        except:
+            if self.espeak:
+                self.CONFIG['google_routesearch_speak'] = 1
+            else:
+                self.CONFIG['google_routesearch_speak'] = 0
+
         #self.ico = "google_routesearch.png"
 
         try:
@@ -287,6 +315,11 @@ class google_routesearch:
         except:
             self.CONFIG['google_routesearch_to'] = ""
 
+        try:
+            print "google_routesearch auto recalculate:", self.CONFIG['google_routesearch_auto_recalculate']
+        except:
+            self.CONFIG['google_routesearch_auto_recalculate'] = 1
+
         try:
             print "google_routesearch mode:", self.CONFIG['google_routesearch_mode']
         except:
@@ -297,6 +330,12 @@ class google_routesearch:
         print "__init__ google routesearch"
 
 
+
+    def espeak_cmd(self, str):
+        print "---- google_routesearch espeak ----"
+        print "---- speak [%s] ----" % str
+        os.system( ("espeak \"%s\""%( str)) )
+
     def makeInit(self):
         pass
 
@@ -305,29 +344,169 @@ class google_routesearch:
 
     def do_draw(self, gpsmap, gdkdrawable):
         if self.osmMapa.osm.props.zoom>self.zoom_minimum:
+
+
             if len( self.markers_icons ) == 0:
                 for p in self.markers:
                     if p[3] <> None:
                         self.markers_icons.append( self.osmMapa.osm.image_add( p[0], p[1], p[3] ) )
 
-            t_lt, t_ln, b_lt, b_ln = self.osmMapa.osm.get_bbox()
-            t_lt, t_ln, b_lt, b_ln = math.degrees(t_lt), math.degrees(t_ln), math.degrees(b_lt), math.degrees(b_ln)
+            if not self.CONFIG['google_routesearch_speak']:
 
-            cr = gdkdrawable.cairo_create()
+                t_lt, t_ln, b_lt, b_ln = self.osmMapa.osm.get_bbox()
+                t_lt, t_ln, b_lt, b_ln = math.degrees(t_lt), math.degrees(t_ln), math.degrees(b_lt), math.degrees(b_ln)
+
+                cr = gdkdrawable.cairo_create()
+
+                for p in self.markers:
+                    lt = p[0]
+                    ln = p[1]
+                    #print "lt:",t_lt,t_ln,"\t c:",lt,ln,"\tb:",b_lt,b_ln
+                    if t_ln < ln and b_ln > ln and t_lt > lt and b_lt < lt:
+                        x,y = self.osmMapa.osm.geographic_to_screen(lt,ln)
+                        self.osmMapa.draw_cloud( cr, x,y, p[2])
 
-            for p in self.markers:
-                lt = p[0]
-                ln = p[1]
-                #print "lt:",t_lt,t_ln,"\t c:",lt,ln,"\tb:",b_lt,b_ln
-                if t_ln < ln and b_ln > ln and t_lt > lt and b_lt < lt:
-                    x,y = self.osmMapa.osm.geographic_to_screen(lt,ln)
-                    self.osmMapa.draw_cloud( cr, x,y, p[2])
         else:
             for i in self.markers_icons:
                 self.osmMapa.osm.image_remove( i )
             self.markers_icons = []
 
 
+        if self.CONFIG['google_routesearch_speak'] and len(self.points) > 0 and self.CONFIG['gpslock']:
+            cr = gdkdrawable.cairo_create()
+            # find close point
+            id_next = 0
+            latc = self.osmMapa.osm.props.latitude
+            lonc = self.osmMapa.osm.props.longitude
+            dis_min = -10
+            id = 0
+            #print "search for close point------------"
+            for p in self.points:
+                dis = self.osmMapa.distance( p[0],p[1], latc, lonc )
+                if dis_min == -10:
+                    dis_min = dis
+                    id_next = id
+
+                elif dis_min>=dis:
+                    dis_min = dis
+                    id_next = id
+
+                    #print "dis_min:[", dis_min,"] id:[",id,"]"
+                id+=1
+
+            #print "next point id:[",id_next,"]"
+            # chk id_next is forword or backword
+            try:
+                p0 = self.points[id_next]
+                p1 = self.points[id_next+1]
+
+                dis_p0_pc = self.osmMapa.distance( p0[0],p0[1], latc, lonc )
+                dis_p1_pc = self.osmMapa.distance( p1[0],p1[1], latc, lonc )
+                dis_p1_p0 = self.osmMapa.distance( p1[0],p1[1], p0[0], p0[1] )
+                #print "p0_pc[",dis_p0_pc,"] p1_pc[",dis_p1_pc,"] p1_p0[",dis_p1_p0,"]"
+                if dis_p0_pc <  dis_p1_pc and dis_p1_p0>dis_p1_pc:
+                    id_next+=1
+                    #print "search for close point correct"
+            except:
+                print "search for close point correct ERROR"
+            #print "next point id:[",id_next,"]"
+            #p = self.points[id_next]
+            #x,y = self.osmMapa.osm.geographic_to_screen(p[0],p[1])
+            #self.osmMapa.draw_cloud( cr, x,y, ("clost point:\ndist:%f\n[%f,%f]"%(dis_min,p[0],p[1])) )
+
+            id_next-=1
+
+            # next key turn
+            #print "search for next key turn, close point:[",id_next,"]"
+
+            search_for_it = -1
+            for i in range(id_next, len(self.points)):
+                p = self.points[i]
+                #print "p:[",p,"]"
+                #x,y = self.osmMapa.osm.geographic_to_screen(p[0],p[1])
+                #self.osmMapa.draw_cloud( cr, x,y, ("id:[%i]\n[%f,%f]"%(i,p[0],p[1])) )
+
+                id = 0
+                for m in self.markers:
+                    #print "marker:[",m,"] p[",p,"]"
+                    if m[0] == p[0] and m[1] == p[1]:
+                        search_for_it = id
+
+                        dis_to_key = self.osmMapa.distance( m[0], m[1], latc, lonc )
+
+                        #x,y = self.osmMapa.osm.geographic_to_screen(m[0],m[1])
+                        #self.osmMapa.draw_cloud( cr, x,y,
+                        #    ("marker:[%f,%f]\nkey id:[%i] point id:[%i]\ndis:%s\n--------------espeak[%s]"%(m[0],m[1],id,i,self.osmMapa.distance_to_string(dis_to_key),m[2])) )
+
+                        #speak = "in %s %s " % ( self.osmMapa.distance_to_string(dis_to_key).replace("m","meters").replace("km", "kilometers"),  m[2])
+                        #print ("marker:[%f,%f]\tkey id:[%i] point id:[%i]\tdis:%s\n---espeak[%s]"%(m[0],m[1],id,i,self.osmMapa.distance_to_string(dis_to_key),speak))
+
+                        t = int( time.strftime("%Y%m%d%H%M%S", time.localtime() ) )
+
+
+                        info = "%s\n%s" % ( self.osmMapa.distance_to_string(dis_to_key), m[2] )
+                        self.osmMapa.draw_cloud( cr, 50,5, info, 20, False)
+                        if m[4]<> None:
+                            ico = cairo.ImageSurface.create_from_png( m[4] )
+                            cr.set_source_surface( ico, 50,65 )
+                            cr.paint()
+
+
+
+                        if self.espeak_say <> m[2]:
+                            self.espeak_say = m[2]
+                            self.espeak_dis = 0.203
+                            self.espeak_time = t
+
+                            speak = "in %s %s " % ( self.osmMapa.distance_to_string(dis_to_key).replace("m","meters").replace("km", "kilometers"),  m[2])
+                            self.espeak_cmd( speak )
+                        elif self.espeak_say == m[2] and t > (self.espeak_time):
+                                #print dis_to_key,"\t",self.espeak_dis,"\t",t
+
+                                if dis_to_key < 0.200 and self.espeak_dis > 0.200:
+                                    self.espeak_dis = dis_to_key
+                                    self.espeak_time = t+2
+
+                                    speak = "in %s %s " % ( self.osmMapa.distance_to_string(dis_to_key).replace("m","meters").replace("km", "kilometers"),  m[2])
+                                    self.espeak_cmd( speak )
+
+                                elif dis_to_key < 0.100 and self.espeak_dis > 0.100:
+                                    self.espeak_dis = dis_to_key
+                                    self.espeak_time = t+2
+
+                                    speak = "in %s %s " % ( self.osmMapa.distance_to_string(dis_to_key).replace("m","meters").replace("km", "kilometers"),  m[2])
+                                    self.espeak_cmd( speak )
+                                elif dis_to_key < 0.025 and self.espeak_dis > 0:
+                                    self.espeak_dis = dis_to_key
+                                    self.espeak_time = t+2
+
+                                    speak = "%s " % ( m[2])
+                                    self.espeak_cmd( speak )
+
+
+
+
+
+
+
+                        """
+                        self.espeak_say = ""
+                        self.espeak_dis = 0
+                        self.espeak_time = 0
+                        """
+
+
+                        #search_for_it = -1
+                        break
+                        #print "marker found !",m
+                    id+=1
+
+                if search_for_it <> -1:
+                    break
+
+
+
+
     def clean_layer(self):
         for i in self.markers_icons:
             self.osmMapa.osm.image_remove( i )
@@ -342,12 +521,13 @@ class google_routesearch:
         self.icons = []
 
         self.markers = []
+        self.my_point = -1
 
     def on_clear(self, w, dialog):
         self.clean_layer()
         dialog.destroy()
 
-    def search_action(self):
+    def search_action(self, dialog=None):
         self.clean_layer()
 
         try:
@@ -373,8 +553,8 @@ class google_routesearch:
         self.CONFIG['google_routesearch_from'] = self.e_start.get_text()
         self.CONFIG['google_routesearch_to'] = self.e_end.get_text()
 
+        self.search_action( dialog )
 
-        self.search_action()
 
     def on_swap(self, w ):
         a = self.e_start.get_text()
@@ -440,17 +620,26 @@ class google_routesearch:
 </trk>
 </gpx>"""         )
             f.close()
+    def on_bt_speak(self, w):
+        self.CONFIG['google_routesearch_speak'] = w.get_active()
 
     def get_menu_widgets(self, dialog):
         vbox = gtk.VBox()
 
+
+        hbox =gtk.HBox()
+        hbox.pack_start(gtk.Label("A:"),False)
         self.e_start = gtk.Entry()
         self.e_start.set_text(self.CONFIG['google_routesearch_from'])
-        vbox.pack_start(self.e_start, True)
+        hbox.pack_start(self.e_start, True)
+        vbox.pack_start(hbox, True)
 
+        hbox =gtk.HBox()
+        hbox.pack_start(gtk.Label("B:"),False)
         self.e_end = gtk.Entry()
         self.e_end.set_text(self.CONFIG['google_routesearch_to'])
-        vbox.pack_start(self.e_end, True)
+        hbox.pack_start(self.e_end, True)
+        vbox.pack_start(hbox, True)
 
 
         hbox = gtk.HBox()
@@ -473,6 +662,23 @@ class google_routesearch:
         vbox.pack_start( hbox, True )
 
 
+        if self.espeak:
+            self.bt_speak = hildon.CheckButton(gtk.HILDON_SIZE_AUTO)
+            self.bt_speak.set_label("speak")
+            if self.CONFIG['google_routesearch_speak']:
+                self.bt_speak.set_active( True )
+            self.bt_speak.connect("clicked", self.on_bt_speak)
+            hbox.pack_start(self.bt_speak, True)
+
+
+        """
+        #sftp://yoyo@yoyo.ghost.pl/home/yoyo/yosmapa_builder/yosmapa_builder
+        self.bt_auto = hildon.CheckButton(gtk.HILDON_SIZE_AUTO)
+        self.bt_auto.set_label("auto")
+        hbox.pack_start(self.bt_offline, True)
+        vbox.pack_start( hbox, True )
+        """
+
         hbox = gtk.HBox()
 
         bt_a = gtk.Button(stock=gtk.STOCK_ADD)
