@@ -56,8 +56,10 @@ gtk.gdk.threads_init()
 
 
 name = "yosmapa"
-version = "0.17"
+version = "0.18"
 """
+0.18
+    plugin[google_routesearch] support for espeak in turn by turn mode :)
 0.17
     on clouds actions:
         center to
@@ -131,6 +133,9 @@ version = "0.17"
 """
 
 
+def deb(str):
+    print "-------------------------\n",str,"\n-------------------------"
+
 
 #Try static lib first
 mydir = os.path.dirname(os.path.abspath(__file__))
@@ -138,12 +143,12 @@ libdir = os.path.join(mydir, ".libs")
 sys.path.insert(0, libdir)
 
 import osmgpsmap
-print "using library: %s" % osmgpsmap.__file__
+deb("using library: %s" % osmgpsmap.__file__)
 
 
 try:
     import location
-    print "gps found!"
+    deb("gps found!")
     gpsPresent = 1
 except Exception, e:
     print "FROM ERROR location ",e
@@ -209,7 +214,7 @@ def config_save():
     f = open(home,"w")
     pickle.dump( ts, f )
     f.close()
-    print "config - save done"
+    print "done"
 
 def config_load():
     global home
@@ -223,7 +228,7 @@ def config_load():
         for b in tp:
             CONFIG[b[0]] = b[1]
 
-        print "config - done"
+        print "done"
     else:
         print "config - config file not found :/"
         config_save()
@@ -811,6 +816,14 @@ class UI(gtk.Window):
             return dist * 111.18957696
         except:
             return 0
+    def distance_to_string(self, dis):
+        if dis > 1.0:
+            return "%.1f km"%dis
+        else:
+            dis*=1000
+            return "%i m"%dis
+
+
     def cal_angle(self, lat0, lon0, lat1, lon1):
         deltax = lat1-lat0
         deltay = lon0-lon1
@@ -878,6 +891,8 @@ class UI(gtk.Window):
                 cr.show_text("%i m"%dis)
 
     def draw_cloud(self, cr, x, y, str, font_size = 13, Fifek = True):
+        if x<0 or y<0:
+            return None
         rlat, rlon = self.osm.get_co_ordinates(int(x), int(y))
         if str == "":
             return None
